package com.spicysoft.sample.dribble;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.RectF;
import android.util.Log;

import com.spicysoft.android.framwork.GameView;
import com.spicysoft.android.framwork.Scene;

abstract class BallAnimationScene implements Scene
{
  /** ~ */
  private static final float PI = 3.14F;
  /** {[Eǂɂ̑x̐̌W */
  private static final float DECAY_VERTICAL   = 0.7F;
  /** {[Eǂɂ̑x̐̌W */
  private static final float DECAY_HORIZONTAL = 0.9F;
  /** {[̕\a */
  private static final float BALL_RADIUS = 32;
  /** {[̃Aj[Ṽp^[(180x) */
  private static final int BALL_ANIMATION = 8;
  /** d͉xFP(`P PER b̓) */
  protected static final float GRAVITY = 2000;
  /** 
   * f[^ς݂ǂH
   *  ̔hV[Ԃŋʂ̃f[^񂵂Ȃ悤
   */
  private static boolean inited = false;
  /** {[̃Aj[V摜̊ep^[̈ʒu */
  private static final Rect[] ballSrc = new Rect[BALL_ANIMATION];
  /** {[\̋` */
  private static final RectF ballDst = new RectF();
  /** {[摜 */
  private static Bitmap ball;
  /** {[摜\Ƃ̕` */
  private static Paint ballPaint;
  /** wi̕` */
  private static Paint backgroundPaint;
  /** {[̈ʒu(XW) */
  private static float ballX;
  /** {[̈ʒu(XW) */
  private static float ballY;
  /** {[̑x(P:`P PER b) */
  protected static float ballVX;
  /** {[̑x(P:`P PER b) */
  protected static float ballVY;
  /** {[̉]px P:0`2PI */
  private static float rotation;
  /** {[̉]x P:px/b */
  private static float rotationV;

  @Override public void init(GameView view)
  {
    if (inited) {
      return;
    }
    ball = BitmapFactory.decodeResource(view.getResources(), R.drawable.ball);
    final int ballSrcHeight = ball.getHeight();
    for (int n = 0; n < BALL_ANIMATION; n++) {
      ballSrc[n] = new Rect(ballSrcHeight * n, 0, ballSrcHeight * (n + 1),
          ballSrcHeight);
    }

    backgroundPaint = new Paint();
    backgroundPaint.setColor(Color.BLACK);

    ballPaint = new Paint();
    inited = true;
  }

  @Override public void start(GameView view)
  {
    ballX = view.getWidth() / 2;
    ballY = 0;
    ballVX = 0;
    ballVY = 0;
    rotation = 0;
    rotationV = - 2 * PI;
  }

  @Override public void process(GameView view) 
  {
    ballX = ballX + ballVX / GameView.FRAMES_PER_SECOND;
    ballY = ballY + ballVY / GameView.FRAMES_PER_SECOND;

    rotation = rotation + rotationV / GameView.FRAMES_PER_SECOND;
    while (rotation < 0) {
      rotation += 2 * PI;
    }
    while (2 * PI <= rotation) {
      rotation -= 2 * PI;
    }

    if (ballX <= BALL_RADIUS) {
      ballX = -(BALL_RADIUS - ballX) * DECAY_VERTICAL * DECAY_VERTICAL + BALL_RADIUS;
      ballVX = -ballVX * DECAY_VERTICAL;
      ballVY = ballVY * DECAY_HORIZONTAL;
      //rotationV = rotationV + ballVY * (1 - DECAY_HORIZONTAL);
    }
    final float rightWall = view.getWidth() - BALL_RADIUS;
    if (rightWall <= ballX) {
      ballX = rightWall- (ballX - rightWall) * DECAY_VERTICAL * DECAY_VERTICAL;
      ballVX = -ballVX * DECAY_VERTICAL;
      ballVY = ballVY * DECAY_HORIZONTAL;
      //rotationV = rotationV + ballVY * (1 - DECAY_HORIZONTAL);
    }
    final float floor = view.getHeight() - BALL_RADIUS;
    if (floor <= ballY) {
      ballY = floor - (ballY - floor) * DECAY_VERTICAL * DECAY_VERTICAL;
      final float limit = GRAVITY / GameView.FRAMES_PER_SECOND * 3;
      if (floor - ballY < 2 && ballVY <= limit) {
        ballY = floor;
        ballVY = 0;
      } else {
        ballVY = -ballVY * DECAY_VERTICAL;
        onDribbled();
        Log.d("Dribble","dribbled:" + ballY + "," + ballVY + "/" + limit);
      }
      ballVX = ballVX * DECAY_HORIZONTAL;
      ballVX = ballVX + rotationV * 32 * (1 - DECAY_HORIZONTAL) * DECAY_HORIZONTAL;
      rotationV = (rotationV + ballVX*(1 - DECAY_HORIZONTAL) / 32) * DECAY_HORIZONTAL;
    } else {
      ballVY += GRAVITY / GameView.FRAMES_PER_SECOND;
    }
  }

  @Override public void draw(Canvas canvas) 
  {
    canvas.drawRect(0, 0, canvas.getWidth(), canvas.getHeight(), backgroundPaint);
    ballDst.set(ballX - BALL_RADIUS, ballY - BALL_RADIUS, ballX + BALL_RADIUS,
        ballY + BALL_RADIUS);

    final int index = ((int)(rotation / (2 * PI) * BALL_ANIMATION * 2)) % BALL_ANIMATION;
    canvas.drawBitmap(ball, ballSrc[index], ballDst, ballPaint);
  }

  public void onDribbled() {
  }
}
